import itertools

import math
import numpy as np
import random

from Sensei.data import load_data_from_file


def print_event_counts(Y_train, frequency):
    segments = []
    for night in Y_train:
        labels = []
        for label in night[0]:
            labels.append(np.where(label == 1)[0][0])

        segments.append(labels)
    apnea = []
    osa = []
    idk = []
    for night in segments:
        values = [list(v) for k, v in itertools.groupby(night)]

        for value in values:
            if value[0] == 0:
                pass
            if value[0] == 1:
                apnea.append(len(value) / frequency)
            elif value[0] == 2:
                osa.append(len(value) / frequency)
            elif value[0] == 3:
                idk.append(len(value) / frequency)
    print("Apnea: ", apnea)
    print("Osa:", osa)
    # print(collections.Counter(idk))


def reduce_data_for_training(X_train, Y_train, X_test, Y_test, size_train, size_test):
    x_train = X_train[:size_train]
    x_test = X_test[:size_test]
    y_train = Y_train[:size_train]
    y_test = Y_test[:size_test]

    return x_train, y_train, x_test, y_test


def keep_only_select_columns(X, columns):
    x = []

    for night in X:
        night_sliced = night[:, columns]
        x.append(night_sliced)

    return x


def load_bergen_and_colosseum_combined(path_bergen, path_colosseum, shuffle_seed):
    X_bergen, Y_bergen = load_data_from_file(path_bergen)
    X_colosseum, Y_colosseum = load_data_from_file(path_colosseum)

    X_bergen.extend(X_colosseum)
    Y_bergen.extend(Y_colosseum)

    random.seed(shuffle_seed)

    combined = list(zip(X_bergen, Y_bergen))
    random.shuffle(combined)
    X_bergen[:], Y_bergen[:] = zip(*combined)

    return X_bergen, Y_bergen


def print_class_count(Y):
    obs = set()
    c = [0] * 3
    for val in Y:
        d = val.data
        c[d] += 1
    print(c)


def split_train_test(X, Y, split):
    train_size = math.floor((1 - split) * len(X))

    x_train, x_test = X[:train_size], X[train_size:]
    y_train, y_test = Y[:train_size], Y[train_size:]

    print("Treningsfiler: ", len(x_train))
    print("Testfiler: ", len(x_test))

    return x_train, x_test, y_train, y_test
