import torch
import torch.nn.functional as F


class SimpleCNN(torch.nn.Module):

    def __init__(self, nr_features, dropout=0.2, channels=30):
        super(SimpleCNN, self).__init__()

        # self.batchnorm1 = torch.nn.BatchNorm1d(num_features=nr_features)

        self.conv1 = torch.nn.Conv1d(nr_features, channels, kernel_size=3, stride=1, padding=1)
        self.batchnorm1 = torch.nn.BatchNorm1d(channels)
        self.pool1 = torch.nn.MaxPool1d(kernel_size=2, stride=1, padding=1)

        self.conv2 = torch.nn.Conv1d(channels, channels, kernel_size=3, stride=1, padding=0)
        self.batchnorm2 = torch.nn.BatchNorm1d(channels)
        self.pool2 = torch.nn.MaxPool1d(kernel_size=2, stride=1, padding=1)

        self.conv3 = torch.nn.Conv1d(channels, channels, kernel_size=3, stride=1, padding=0)
        self.batchnorm3 = torch.nn.BatchNorm1d(channels)
        self.pool3 = torch.nn.MaxPool1d(kernel_size=2, stride=1, padding=1)

        self.conv4 = torch.nn.Conv1d(channels, channels, kernel_size=2, stride=1, padding=0)
        self.batchnorm4 = torch.nn.BatchNorm1d(channels)

        self.conv5 = torch.nn.Conv1d(channels, channels, kernel_size=2, stride=1, padding=0)
        self.batchnorm5 = torch.nn.BatchNorm1d(channels)

        self.conv6 = torch.nn.Conv1d(channels, channels, kernel_size=2, stride=1, padding=0)
        self.batchnorm6 = torch.nn.BatchNorm1d(channels)

        nr_nodes_needed = channels * 143

        self.fc1 = torch.nn.Linear(nr_nodes_needed, 256)
        self.fc2 = torch.nn.Linear(256, 128)
        self.fc3 = torch.nn.Linear(128, 64)
        self.fc4 = torch.nn.Linear(64, 3)

        self.dropout = torch.nn.Dropout(p=dropout)

    def forward(self, x):
        batch_size = x.size()[0]

        # Computes the activation of the first convolution
        x = self.conv1(x)
        x = F.relu(x)
        x = self.pool1(x)
        x = self.batchnorm1(x)

        x = self.conv2(x)
        x = F.relu(x)
        x = self.pool2(x)
        x = self.batchnorm1(x)

        x = self.conv3(x)
        x = F.relu(x)
        x = self.pool3(x)
        x = self.batchnorm1(x)

        x = self.conv4(x)
        x = F.relu(x)

        x = self.conv4(x)
        x = F.relu(x)

        x = self.conv4(x)
        x = F.relu(x)

        x = self.conv4(x)
        x = F.relu(x)
        # x = self.batchnorm4(x)

        x = self.conv5(x)
        x = F.relu(x)
        # x = self.batchnorm5(x)

        x = self.conv6(x)
        x = F.relu(x)
        # x = self.batchnorm6(x)

        # Reshape data to input to the input layer of the neural net
        x = x.view(batch_size, -1)
        # print(x.size())

        # Computes the activation of the first fully connected layer
        x = self.dropout(x)
        x = torch.sigmoid(self.fc1(x))
        x = self.dropout(x)
        x = torch.sigmoid(self.fc2(x))
        x = torch.sigmoid(self.fc3(x))
        x = self.fc4(x)

        return x


class SimpleCNNDeep(torch.nn.Module):

    def __init__(self, nr_features, dropout=0.2, channels=30):
        super(SimpleCNNDeep, self).__init__()

        # self.batchnorm1 = torch.nn.BatchNorm1d(num_features=nr_features)

        self.conv1 = torch.nn.Conv1d(nr_features, channels, kernel_size=3, stride=1, padding=1)
        self.batchnorm1 = torch.nn.BatchNorm1d(channels)
        self.pool1 = torch.nn.MaxPool1d(kernel_size=2, stride=1, padding=1)

        self.conv2 = torch.nn.Conv1d(channels, channels, kernel_size=3, stride=1, padding=0)
        self.batchnorm2 = torch.nn.BatchNorm1d(channels)
        self.pool2 = torch.nn.MaxPool1d(kernel_size=2, stride=1, padding=1)

        self.conv3 = torch.nn.Conv1d(channels, channels, kernel_size=3, stride=1, padding=0)
        self.batchnorm3 = torch.nn.BatchNorm1d(channels)
        self.pool3 = torch.nn.MaxPool1d(kernel_size=2, stride=1, padding=1)

        self.conv4 = torch.nn.Conv1d(channels, channels, kernel_size=3, stride=1, padding=0)
        self.batchnorm4 = torch.nn.BatchNorm1d(channels)

        self.conv5 = torch.nn.Conv1d(channels, channels, kernel_size=4, stride=1, padding=0)
        self.batchnorm5 = torch.nn.BatchNorm1d(channels)

        self.conv6 = torch.nn.Conv1d(channels, channels, kernel_size=4, stride=1, padding=0)
        self.batchnorm6 = torch.nn.BatchNorm1d(channels)

        nr_nodes_needed = channels * 125

        self.fc1 = torch.nn.Linear(nr_nodes_needed, 256)
        self.fc2 = torch.nn.Linear(256, 128)
        self.fc3 = torch.nn.Linear(128, 64)
        self.fc4 = torch.nn.Linear(64, 3)

        self.dropout = torch.nn.Dropout(p=dropout)

    def forward(self, x):
        batch_size = x.size()[0]

        # Computes the activation of the first convolution
        x = self.conv1(x)
        x = F.relu(x)
        x = self.pool1(x)
        x = self.batchnorm1(x)

        x = self.conv2(x)
        x = F.relu(x)
        x = self.pool2(x)
        x = self.batchnorm1(x)

        x = self.conv3(x)
        x = F.relu(x)
        x = self.pool3(x)
        x = self.batchnorm1(x)

        x = self.conv4(x)
        x = F.relu(x)
        x = self.batchnorm1(x)

        x = self.conv4(x)
        x = F.relu(x)
        x = self.batchnorm1(x)

        x = self.conv4(x)
        x = F.relu(x)
        x = self.batchnorm1(x)

        x = self.conv4(x)
        x = F.relu(x)
        x = self.batchnorm1(x)

        x = self.conv4(x)
        x = F.relu(x)
        x = self.batchnorm1(x)

        x = self.conv4(x)
        x = F.relu(x)

        x = self.conv4(x)
        x = F.relu(x)

        x = self.conv4(x)
        x = F.relu(x)

        x = self.conv4(x)
        x = F.relu(x)
        # x = self.batchnorm4(x)

        x = self.conv5(x)
        x = F.relu(x)
        # x = self.batchnorm5(x)

        x = self.conv6(x)
        x = F.relu(x)
        # x = self.batchnorm6(x)

        # Reshape data to input to the input layer of the neural net
        x = x.view(batch_size, -1)
        # print(x.size())

        # Computes the activation of the first fully connected layer
        x = self.dropout(x)
        x = torch.sigmoid(self.fc1(x))
        x = self.dropout(x)
        x = torch.sigmoid(self.fc2(x))
        x = torch.sigmoid(self.fc3(x))
        x = self.fc4(x)

        return x


class CNN_LSTM(torch.nn.Module):

    def __init__(self, nr_features):
        super(CNN_LSTM, self).__init__()

        # self.batchnorm1 = torch.nn.BatchNorm1d(num_features=nr_features)

        self.conv1 = torch.nn.Conv1d(nr_features, 15, kernel_size=3, stride=1, padding=1)
        self.batchnorm1 = torch.nn.BatchNorm1d(15)
        self.pool1 = torch.nn.MaxPool1d(kernel_size=2, stride=1, padding=1)

        self.conv2 = torch.nn.Conv1d(15, 15, kernel_size=3, stride=1, padding=1)
        self.batchnorm2 = torch.nn.BatchNorm1d(15)
        self.pool2 = torch.nn.MaxPool1d(kernel_size=2, stride=1, padding=1)

        self.conv3 = torch.nn.Conv1d(15, 15, kernel_size=3, stride=1, padding=1)
        self.batchnorm3 = torch.nn.BatchNorm1d(15)
        self.pool3 = torch.nn.MaxPool1d(kernel_size=2, stride=1, padding=1)

        self.conv4 = torch.nn.Conv1d(15, 15, kernel_size=2, stride=1, padding=0)
        self.batchnorm4 = torch.nn.BatchNorm1d(15)

        self.conv5 = torch.nn.Conv1d(15, 15, kernel_size=2, stride=1, padding=0)
        self.batchnorm5 = torch.nn.BatchNorm1d(15)

        self.conv6 = torch.nn.Conv1d(15, 15, kernel_size=2, stride=1, padding=0)
        self.conv6Linear = torch.nn.Linear(150, 1920)

        self.batchNormLSTM1 = torch.nn.BatchNorm1d(15)
        self.dropoutLSTM1 = torch.nn.Dropout(p=0.2)
        self.lstm1 = torch.nn.LSTM(input_size=10, hidden_size=64, num_layers=1, bidirectional=True)

        # self.batchNormLSTM2 = torch.nn.BatchNorm1d(15)
        # self.dropoutLSTM2 = torch.nn.Dropout(p=0.2)
        # self.lstm2 = torch.nn.LSTM(input_size=512, hidden_size=256, num_layers=1, bidirectional=True)

        # self.batchNormLSTM3 = torch.nn.BatchNorm1d(15)
        # self.dropoutLSTM3 = torch.nn.Dropout(p=0.2)
        # self.lstm3 = torch.nn.LSTM(input_size=512, hidden_size=256, num_layers=1, bidirectional=True)

        self.fc5 = torch.nn.Linear(1920, 256)
        self.dropout = torch.nn.Dropout(p=0.2)
        self.fc6 = torch.nn.Linear(256, 64)
        self.fc7 = torch.nn.Linear(64, 3)

    def forward(self, x):
        batch_size = x.size()[0]
        # print(x.size())
        # Size of X: [256, 7, 10] [ Batch, features, length

        x = self.conv1(x)
        x = self.batchnorm1(x)
        x = F.relu(x)
        x = self.pool1(x)
        # print(x.size())
        # Size of X: [256, 15, 6] [ Batch, features, length

        x = self.conv2(x)
        x = self.batchnorm2(x)
        x = F.relu(x)
        x = self.pool2(x)
        # print(x.size())
        # Size of X: [256, 15, 7] [ Batch, features, length

        x = self.conv3(x)
        x = self.batchnorm3(x)
        x = F.relu(x)
        x = self.pool3(x)
        # print(x.size())
        # Size of X: [256, 15, 8] [ Batch, features, length

        x = self.conv4(x)
        x = self.batchnorm4(x)
        x = F.relu(x)
        # print(x.size())
        # Size of X: [256, 15, 9] [ Batch, features, length

        x = self.conv5(x)
        x = self.batchnorm5(x)
        x = F.relu(x)
        # print(x.size())
        # Size of X: [256, 15, 10] [ Batch, features, length

        x = self.conv6(x)
        x = F.relu(x)

        r = x.view(batch_size, -1)
        r2 = self.conv6Linear(r)
        # print(r.size())
        # print(r2.size())

        # print(x.size())
        # print("a")
        # Size of X: [256, 15, 11] [ Batch, features, length

        x = self.batchNormLSTM1(x)
        x = self.dropoutLSTM1(x)
        x, hidden = self.lstm1(x)

        # print("lstm1" + str(x.size()))

        # print("b")
        x = x.view(batch_size, -1)

        # print(x.size())
        # print("A")

        ratio = 1
        x = r2 * ratio + x * (ratio - 1)

        # print(x.size())

        # x = self.batchNormLSTM2(x)
        # print(x.size())
        # print("c")

        # x = self.dropoutLSTM2(x)
        # print(x.size())
        # print("d")

        # x, hidden = self.lstm2(x)
        # print(x.size())
        # print("e")

        # x = self.batchNormLSTM3(x)
        # x = self.dropoutLSTM3(x)
        # x, hidden = self.lstm3(x)

        x = x.view(batch_size, -1)
        # Reshape data to input to the input layer of the neural net
        # print(x.size())

        x = F.sigmoid(self.fc5(x))
        x = self.dropout(x)
        x = F.sigmoid(self.fc6(x))
        x = self.fc7(x)

        return x
